package be.jeffcheasey88.peeratcode.parser.java;

import java.util.ArrayList;
import java.util.List;

import be.jeffcheasey88.peeratcode.parser.Token;
import be.jeffcheasey88.peeratcode.parser.java.Annotation.AnnotableBuffer;
import be.jeffcheasey88.peeratcode.parser.java.Function.FunctionContainer;
import be.jeffcheasey88.peeratcode.parser.java.Variable.VariableContainer;

interface ClassContainer{
	
	void addClass(Class clazz);
	
}

public class Class extends JavaElement implements AnnotableBuffer, ClassContainer, FunctionContainer, VariableContainer{
	
	private List<Annotation> annotationBuffer;
	
	private List<Annotation> annotations;
	
	private Token name;
	private Token extend;
	private Token implement;
	
	private List<JavaElement> elements;
	
	public Class(List<Annotation> annotations, Token name){
		this.annotations = annotations;
		this.name = name;
		this.elements = new ArrayList<>();
	}
	
	public Class(List<Annotation> annotations, Token name, Token extend, Token implement){
		this(annotations, name);
		this.extend = extend;
		this.implement = implement;
	}

	@Override
	public void addFunction(Function function) {
		this.elements.add(function);
	}

	@Override
	public void addVariable(Variable variable) {
		this.elements.add(variable);
	}

	@Override
	public void addClass(Class clazz){
		this.elements.add(clazz);
	}

	@Override
	public <E extends JavaElement> E find(java.util.function.Function<JavaElement, Boolean> finder){
		for(Annotation annotation : this.annotations){
			if(finder.apply(annotation)) return (E) annotation;
		}
		for(JavaElement element : this.elements){
			if(finder.apply(element)) return (E) element;
		}
		return null;
	}

	@Override
	public void addAnnotationBuffer(Annotation annotation){
		if(annotationBuffer == null) annotationBuffer = new ArrayList<>();
		annotationBuffer.add(annotation);
	}
	
	public List<Annotation> getAnnotationBuffer(){
		List<Annotation> list = this.annotationBuffer;
		this.annotationBuffer = null;
		return list;
	}
	

}
